@@ -2,14 +2,14 @@ package Dancer2::Plugin::Auth::Extensible::Provider::ActiveDirectory;
 
 use feature qw/state/;
 use Carp qw/croak/;
-use Dancer2::Core::Types qw/HashRef Str Int/;
+use Dancer2::Core::Types qw/HashRef Str Int ArrayRef/;
 use Auth::ActiveDirectory;
 
 use Moo;
 with "Dancer2::Plugin::Auth::Extensible::Role::Provider";
 use namespace::clean;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 NAME
 
@@ -90,10 +90,25 @@ has principal => (
     required => 1,
 );
 
+=head2 allowed_groups
+
+List of groups allowed to login. If empty, all groups are allowed.
+
+=cut
+
+has allowed_groups => (
+    is      => 'ro',
+    isa     => ArrayRef,
+    default => sub { return [] },
+);
+
 =head1 METHODS
 
 =head2 authenticate_user $username, $password
 
+Returns true if the user could be authenticated. Returns also false if the user could be
+authenticated, but is member of any allowed group.
+
 =cut
 
 sub authenticate_user {
@@ -104,11 +119,24 @@ sub authenticate_user {
 
     my $ad = $self->_active_directory or return;
 
-    my $user = $ad->authenticate( $username, $password );
+    my $user = $ad->authenticate( $username, $password ) || return;
 
-    $self->_user_cache($username, $user);
+    my $allowed = 1;
+    if ( @{ $self->allowed_groups } ) {
+        $allowed = 0;
+        for my $allowed_group ( @{ $self->allowed_groups } ) {
+            if (grep { $_ eq $allowed_group } map { $_->name } @{ $user->groups } ) {
+                $allowed = 1;
+                last;
+            }
+        }
+    }
 
-    return !!$user;
+    if ($allowed) {
+        $self->_user_cache($username, $user);
+        return 1;
+    }
+    return;
 }
 
 =head2 get_user_details $username
